@@ -1,3 +1,5 @@
+from copy import copy
+
 import json
 import math
 import multiprocessing
@@ -11,6 +13,7 @@ from hpccm.building_blocks import (
     generic_autotools,
     generic_cmake,
     hdf5,
+    mkl,
     packages,
     pip,
     scif,
@@ -20,7 +23,7 @@ from ogscm.building_blocks.pm_conan import pm_conan
 import hpccm
 from ogscm.building_blocks.paraview import paraview
 from ogscm.building_blocks.ccache import ccache
-from hpccm.primitives import comment, environment, raw
+from hpccm.primitives import comment, copy, environment, raw, shell
 from hpccm import linux_distro
 import os
 from ogscm.building_blocks.ogs import ogs
@@ -104,6 +107,7 @@ parse_g.add_argument(
     action="store_true",
     help="Installs development tools (vim, gdb)",
 )
+parse_g.add_argument("--mkl", dest="mkl", action="store_true", help="Use MKL. By setting this option, you agree to the [Intel End User License Agreement](https://software.intel.com/en-us/articles/end-user-license-agreement).")
 
 # Parse local args
 local_args = parser.parse_known_args()[0]
@@ -169,7 +173,7 @@ if local_args.ogs not in ["off", "clean"]:  # != "off" and local_args.ogs != "cl
         else:
             name_start = f"ogs-{commit_hash[:8]}"
 
-folder = f"/{name_start}/{local_args.pm}"
+folder = f"/{name_start}/{local_args.pm}".replace("//", "/")
 
 if len(local_args.cmake_args) > 0:
     cmake_args_hash = hashlib.md5(
@@ -227,15 +231,15 @@ if local_args.gui:
     )
 if local_args.ogs != "clean":
     if local_args.pm == "conan":
-        Stage0 += cmake(eula=True, version="3.16.6")
+        Stage0 += cmake(eula=True, version="3.19.4")
         conan_user_home = "/opt/conan"
         if local_args.dev:
             conan_user_home = ""
         Stage0 += pm_conan(user_home=conan_user_home)
         Stage0 += environment(variables={"CONAN_SYSREQUIRES_SUDO": 0})
     elif local_args.pm == "system":
-        Stage0 += cmake(eula=True, version="3.16.6")
-        Stage0 += boost(version="1.66.0", bootstrap_opts=["headers"])
+        Stage0 += cmake(eula=True, version="3.19.4")
+        Stage0 += boost(version="1.67.0", bootstrap_opts=["headers"])
         Stage0 += environment(variables={"BOOST_ROOT": "/usr/local/boost"})
         Stage0 += packages(apt=["libxml2-dev"], yum=["libxml2-devel"])
         vtk_cmake_args = [
@@ -250,17 +254,11 @@ if local_args.ogs != "clean":
                     "libgeotiff-dev",
                     "libshp-dev",
                     "libnetcdf-c++4-dev",
-                    "libqt5x11extras5-dev",
-                    "libqt5xmlpatterns5-dev",
-                    "qt5-default",
                 ],
                 yum=[
                     "libgeotiff-devel",
                     "shapelib-devel",
                     "netcdf-devel",
-                    "qt5-qtbase-devel",
-                    "qt5-qtxmlpatterns-devel",
-                    "qt5-qtx11extras-devel",
                 ],
             )
             Stage1 += packages(
@@ -268,19 +266,50 @@ if local_args.ogs != "clean":
                     "geotiff-bin",
                     "shapelib",
                     "libnetcdf-c++4",
-                    "libqt5x11extras5",
-                    "libqt5xmlpatterns5",
-                    "qt5-default",
+                    "libglib2.0-0",
+                    "libdbus-1-3",
+                    "libexpat1",
+                    "libfontconfig1",
+                    "libfreetype6",
+                    "libgl1-mesa-glx",
+                    "libglib2.0-0",
+                    "libx11-6",
+                    "libx11-xcb1",
+                    "libxkbcommon-x11-0",
                 ],
+                # TODO: Add runtime packages for centos
                 yum=[
                     "libgeotiff",
                     "shapelib",
                     "netcdf",
-                    "qt5-qtbase",
-                    "qt5-qtxmlpatterns",
-                    "qt5-qtx11extras",
                 ],
             )
+            # TODO: will not work with clang
+            qt_install_dir = "/opt/qt"
+            qt_version = "5.14.2"
+            qt_dir = f"{qt_install_dir}/{qt_version}/gcc_64"
+            Stage0 += pip(pip="pip3", packages=["aqtinstall"])
+            Stage0 += shell(
+                commands=[
+                    f"aqt install --outputdir {qt_install_dir} {qt_version} linux desktop -m xmlpatterns,x11extras"
+                ]
+            )
+            Stage1 += copy(_from="0", src=qt_install_dir, dest=qt_install_dir)
+            Stage0 += environment(
+                variables={
+                    "LD_LIBRARY_PATH": f"{qt_dir}/lib:$LD_LIBRARY_PATH",
+                    "PATH": f"{qt_dir}/bin:$PATH",
+                    "QTDIR": qt_dir,
+                }
+            )
+            Stage1 += environment(
+                variables={
+                    "LD_LIBRARY_PATH": f"{qt_dir}/lib:$LD_LIBRARY_PATH",
+                    "PATH": f"{qt_dir}/bin:$PATH",
+                    "QTDIR": qt_dir,
+                }
+            )
+
             vtk_cmake_args = [
                 "-DVTK_BUILD_QT_DESIGNER_PLUGIN=OFF",
                 "-DVTK_Group_Qt=ON",
@@ -350,9 +379,9 @@ if local_args.ogs != "clean":
                 "Eigen3_ROOT": "/usr/local/eigen",
                 "Eigen3_DIR": "/usr/local/eigen",
             },
-            directory="eigen-3.3.7",
+            directory="eigen-3.3.9",
             prefix="/usr/local/eigen",
-            url="https://gitlab.com/libeigen/eigen/-/archive/3.3.7/eigen-3.3.7.tar.gz",
+            url="https://gitlab.com/libeigen/eigen/-/archive/3.3.9/eigen-3.3.9.tar.gz",
         )
         Stage0 += hdf5(configure_opts=["--enable-cxx"], toolchain=toolchain)
 if local_args.cvode:
@@ -396,6 +425,10 @@ if local_args.tfel:
     )
     Stage0 += environment(variables={"TFELHOME": "/usr/local/tfel"})
 
+if local_args.mkl:
+    Stage0 += mkl(eula=True)
+    cmake_args.append("-DOGS_USE_MKL=ON")
+
 if local_args.ccache:
     Stage0 += ccache(cache_size="15G")
 if local_args.ogs != "off" and local_args.ogs != "clean":
